import json

from autogen_agentchat.messages import TextMessage
from fastapi import APIRouter, HTTPException
from fastapi.responses import StreamingResponse
from loguru import logger
from pydantic import BaseModel

from ..agents.ag.team_builder import TeamBuilder
from ..agents.ag.team_runner import TeamRunner
from ..gomtmclients.rest.models.chat_req import ChatReq

router = APIRouter()


async def run_stream(task: str):
    try:
        team_builder = TeamBuilder()
        team = await team_builder.create_demo_team()
        team_runner = TeamRunner()

        async for event in team_runner.run_stream(
            task=task, team_config=team.dump_component()
        ):
            if isinstance(event, TextMessage):
                yield f"2:{event.model_dump_json()}\n"
            # elif isinstance(event, ToolCallRequestEvent):
            #     yield f"0:{json.dumps(obj=jsonable_encoder(event.content))}\n"
            # elif isinstance(event, TeamResult):
            #     yield f"0:{json.dumps(obj=event.model_dump_json())}\n"

            elif isinstance(event, BaseModel):
                yield f"2:{event.model_dump_json()}\n"
            else:
                yield f"2:{json.dumps(f'unknown event: {str(event)},type:{type(event)}')}\n"
    except Exception as e:
        logger.exception("Streaming error")
        yield f"2:{json.dumps({'error': str(e)})}\n"


@router.api_route(path="/tenants/{tenant}/chat", methods=["GET", "POST"])
async def chat(r: ChatReq):
    try:
        user_messages = r.messages
        if len(user_messages) == 0:
            raise HTTPException(status_code=400, detail="No messages provided")
        task = user_messages[-1].content

        return StreamingResponse(
            content=run_stream(task), media_type="text/event-stream"
        )

    except Exception as e:
        logger.exception("Chat error")
        raise HTTPException(status_code=500, detail=str(e))


# @router.api_route(path="/test_m1", methods=["GET", "POST"])
# async def test_m1(r: Request):
#     from autogen_ext.agents.web_surfer import PlaywrightController

#     # 测试 megentic one agent
#     try:
#         model_client = get_oai_Model()
#         logging_client = LoggingModelClient(model_client)

#         assistant = AssistantAgent(
#             "Assistant",
#             model_client=logging_client,
#         )

#         surfer = PlaywrightController(
#             downloads_folder=".vol/WebSurfer",
#             model_client=model_client,
#         )

#         team = MagenticOneGroupChat([surfer], model_client=logging_client)
#         await Console(team.run_stream(task="用中文写一段关于马克龙的新闻"))

#     except Exception as e:
#         logger.error("Chat error", error=str(e))
#         return {"error": str(e)}
#         return {"error": str(e)}
#         return {"error": str(e)}
#         return {"error": str(e)}
#         return {"error": str(e)}
#         return {"error": str(e)}



import structlog
from fastapi import APIRouter
from fastapi.openapi.docs import get_redoc_html, get_swagger_ui_oauth2_redirect_html

# from mtmlib.decorators.mtform.mtform import MtForm
from opentelemetry import trace

from mtmai.core.config import settings

# from mtmai.models.models import Message

# from mtmai.types.types import Theme
# from mtmai.utils import generate_test_email, send_email

tracer = trace.get_tracer_provider().get_tracer(__name__)
LOG = structlog.get_logger()


# mimetypes.add_type("application/javascript", ".js")
# mimetypes.add_type("text/css", ".css")


router = APIRouter()


@router.get("/health", include_in_schema=False)
async def health_check():
    with tracer.start_as_current_span("health-span"):
        LOG.info("get /health")
        current_span = trace.get_current_span()
        current_span.add_event("This is a span event")
        LOG.warning("This is a log message")
        return {"health": True}


@router.get("/swagger-ui-oauth2-redirect", include_in_schema=False)
async def swagger_ui_redirect():
    return get_swagger_ui_oauth2_redirect_html()


@router.get("/redoc", include_in_schema=False)
async def redoc_html():
    return get_redoc_html(
        openapi_url=router.app().openapi_url,
        title=router.app().title + " - ReDoc",
        redoc_js_url="https://unpkg.com/redoc@next/bundles/redoc.standalone.js",
    )


@router.get("/info", include_in_schema=False)
async def app_info():
    return {
        "app_name": settings.app_name,
        "admin_email": settings.admin_email,
        "items_per_user": settings.items_per_user,
    }


# @router.post(
#     "/test-email/",
#     dependencies=[Depends(get_current_active_superuser)],
#     status_code=201,
#     include_in_schema=False,
# )
# class TestUrlReq(BaseModel):
#     url: str


# class TypesResponse(BaseModel):
#     """
#     如果使用openapi 生成前端代码，缺少了某些类型，请在这里补充
#     """

#     thread_form: ThreadForm | None = None
#     uiState: ChatBotUiStateResponse | None = None
#     thread_ui_state: ThreadUIState | None = None
#     chat_profile: ChatProfile | None = None
#     list_view_props: ListViewProps | None = None
#     # mt_form: MtForm | None = None
#     open_canvas_state: OpenCanvasState | None = None


# @router.get("/types", include_in_schema=True, response_model=TypesResponse)
# async def types():
#     """
#     无实际功能，仅用于openapi生成前端代码
#     """
#     return TypesResponse()
